/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
__IMPORT__
	(
	'chrome://mozlib/content/lib/js/datetime/',
	'formatter.js'
	);


//@@@@@@@@@@@@@@@@
function ASRDF_TripleParser		//<c><a>protected<d>Triple line parser class
(
_propertyParser	//<p>ASRDF_PropertyParser<d>Propert parser
){
//================
this.ASRDF_TripleParser = function		//<m>void<a>private<d>class constructor
(
propertyParser	//<p>string[]<d>Properties key and value --optional
){
this.className	= 'ASRDF_TripleParser';
this.propertyParser = propertyParser;
//i12n
this.outlineParser	= new ASRDF_OutlineParser();
this.formatter		= new DATETIME_Formatter();
this.tripleSet		= [];
this.outlineSet		= [];
this.outlineSet.push(this.getOutlineSlot());
}//</m>ASRDF_TripleParser



//================
this.setIn = function		//<m>int<a>protected<d>Parse a source line
(
line		//<p>string<d>Triple line
){
var
/*Object*/processor = this.outlineParser.getProcessor(line);
if(processor.code == this.K.ERROR.NONE){
	var
	/*string*/method,
	/*string*/resource = ((processor.step != this.K.STEP.OBJECT) ||
		(processor.resource.match(this.K.RSC_PATTERN.OBJECT)) ||
		(processor.resource.match(this.K.RSC_PATTERN.CONTAINER))) ?
		this.propertyParser.getExtentedDeclaration(processor.resource) : processor.resource;
	switch(processor.step){
		case this.K.STEP.SUBJECT:
			method = 'setSubject';
			break;
		case this.K.STEP.PREDICATE:
			method = 'setPredicate';
			break;
		case this.K.STEP.OBJECT:
			method = 'setObject';
			break;
		}	//---switch
	if(method) this[method](resource, processor.level);
	}
return processor.code;
}//</m>setIn



//================
this.setSubject = function		//<m>void<a>private<d>Set up resource as subject in tripleSet member
(
resource,		//<p>string<d>Resource value
raising			//<p>boolean<d>To upper level setting flag
){
if(!raising) this.setLowerLevel(0);		//subject is always at level 0
var
/*Object*/outlineSet = this.outlineSet[this.outlineSet.length - 1],
/*Object*/outlineSlot = this.getContainer(resource) ||
	{
	resource:	resource,
	type:		this.K.RSC_TYPE.RESOURCE
	};
if(!(outlineSlot.resource in this.tripleSet)){
	this.tripleSet[outlineSlot.resource] = [];
	this.tripleSet[outlineSlot.resource][this.K.SLOT.SUBJECT] =
		{
		resource:	outlineSlot.resource,
		type:		outlineSlot.type
		};
	}
outlineSet.subject.resource = outlineSlot.resource;
outlineSet.subject.type = outlineSlot.type;
}//</m>setSubject



//================
this.setPredicate = function		//<m>void<a>private<d>Set up resource as predicate in tripleSet member
(
resource,		//<p>string<d>Resource value
level			//<p>int<d>Outline level
){
if(this.testToUpperLevel(level)) this.setUpperLevel(level);
else this.setLowerLevel(level);
var
/*Object*/outlineSet = this.outlineSet[this.outlineSet.length - 1];
if(!(resource in this.tripleSet[outlineSet.subject.resource])){
	this.tripleSet[outlineSet.subject.resource][resource] = [];
	this.tripleSet[outlineSet.subject.resource][resource][this.K.SLOT.PREDICATE] =
		{
		resource:	resource,
		type:		this.K.RSC_TYPE.RESOURCE
		};
	}
outlineSet.predicate.resource = resource;
}//</m>setPredicate



//================
this.setObject = function		//<m>void<a>private<d>Set up resource as object in tripleSet member
(
resource,		//<p>string<d>Resource value
level			//<p>int<d>Outline level
){
this.setLowerLevel(level);
var
/*Object*/outlineSet = this.outlineSet[this.outlineSet.length - 1],
/*Object*/outlineSlot = this.getContainer(resource);
if(!outlineSlot){
	outlineSlot = this.getCollection(resource);
	if(outlineSlot) this.tripleSet[outlineSet.subject.resource][outlineSet.predicate.resource][this.K.SLOT.PREDICATE].type = this.K.RSC_TYPE.COLLECTION;
	else{
		outlineSlot = this.getResource(resource);
		if(!outlineSlot){
			outlineSlot = this.getLiteral(resource);
			if(this.testMultilineLiteralType(outlineSlot.type)){
				outlineSet.object.resource = (this.testMultilineLiteralType(outlineSet.object.type)) ?
					outlineSet.object.resource + outlineSlot.resource : outlineSlot.resource;		//concatenate with previous line
				outlineSet.object.type = outlineSlot.type;
				return;
				}
			}
		}
	}
var
/*Object*/literalSet = null;
if(this.testLiteralType(outlineSlot.type)){
	if(this.testMultilineLiteralType(outlineSet.object.type)) outlineSlot.resource = outlineSet.object.resource + outlineSlot.resource;		//concatenate with previous line
	literalSet = this.getLiteralSet(outlineSlot.resource);
	outlineSlot.resource = literalSet.resource;
	}
this.tripleSet[outlineSet.subject.resource][outlineSet.predicate.resource][outlineSlot.resource] =
	{
	resource:	outlineSlot.resource,
	type:		outlineSlot.type,
	literalSet:	literalSet
	};
outlineSet.object.resource = outlineSlot.resource;
outlineSet.object.type = outlineSlot.type;
}//</m>setObject



//================
this.setUpperLevel = function		//<m>void<a>private<d>Step to upper subject level
(
level			//<p>int<d>Outline level
){
var
/*string*/object = this.outlineSet[this.outlineSet.length - 1].object;
this.outlineSet.push(this.getOutlineSlot());
const RAISING = true;
this.setSubject(object.resource, RAISING);
}//</m>setUpperLevel



//================
this.setLowerLevel = function		//<m>void<a>private<d>Step to lower subject level
(
level			//<p>int<d>Outline level
){
var
/*int*/outlineSetLevel = this.outlineSet.length - 1;
while(outlineSetLevel-- > level) this.outlineSet.pop();
}//</m>setLowerLevel



//================	REMOVE
this.setKey = function		//<m>void<a>private<d>Set up property key and value replacing key references in value
(
key,		//<p>string<d>Key
value		//<p>string<d>Value
){
var
/*string[]*/valueSet = value.split(this.K.patternSet.KEY_SEPARATOR);
for(var i=0,m=valueSet.length;i<m;i++) if(valueSet[i] in this.tripleSet) valueSet[i] = this.tripleSet[valueSet[i]];
this.tripleSet[key] = valueSet.join('');
}//</m>setKey



//================
this.get_tripleSet = function		//<m>string<a>protected<d>Get tripleSet member
(){
return this.tripleSet;
}//</m>get_tripleSet



//================
this.getContainer = function		//<m>Object<a>private<d>Get container outlineSlot
(
resource		//<p>string<d>Resource value
){
	/****************DEBUG***************/
	//Stdout.setOut('resource : %N', resource);
	/****************DEBUG***************/
var
/*Object*/outlineSlot =
	{
	resource:	'',
	type:		this.K.RSC_TYPE.UNKNOWN
	},
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.CONTAINER);
if(!resourceSet) return null;
switch(resourceSet[1]){
	case 'Bag':
		outlineSlot.type = this.K.RSC_TYPE.BAG;
		break;
	case 'Seq':
		outlineSlot.type = this.K.RSC_TYPE.SEQ;
		break;
	case 'Alt':
		outlineSlot.type = this.K.RSC_TYPE.ALT;
		break;
	}	//---switch
if(this.testBNodeResource(resourceSet[2])){
	outlineSlot.resource = this.getContainerId(resourceSet[1]) + this.getBNodeResource();
	outlineSlot.type |= this.K.RSC_TYPE.BNODE;
	}
else outlineSlot.resource = resource;
return outlineSlot;
}//</m>getContainer



//================
this.getContainerId = function		//<m>string<a>private<d>Get container identifier
(
type		//<p>string<d>Container type
){
return sprintf(this.K.RSC_PATTERN.CONTAINER_ID, type);
}//</m>getContainerId



//================
this.getCollection = function		//<m>Object<a>private<d>Get collection outlineSlot
(
resource		//<p>string<d>Resource value
){
return !resource.match(this.K.RSC_PATTERN.COLLECTION) ? null :
	{
	resource:	'Collection',
	type:		this.K.RSC_TYPE.COLLECTION
	};
}//</m>getCollection



//================
this.getResource = function		//<m>Object<a>private<d>Get resource outlineSlot
(
resource		//<p>string<d>Resource value
){
var
/*Object*/outlineSlot =
	{
	resource:	'',
	type:		this.K.RSC_TYPE.RESOURCE
	},
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.OBJECT);
if(!resourceSet) return null;
/* FIXME: REMOVE
if(this.testBNodeResource(resourceSet[1])){
	outlineSlot.resource = this.getBNodeResource();
	outlineSlot.type |= this.K.RSC_TYPE.BNODE;
	}
else outlineSlot.resource = resourceSet[1];
*/
outlineSlot.resource = resourceSet[1];
return outlineSlot;
}//</m>getResource



//================
this.getLiteral = function		//<m>Object<a>private<d>Get literal outlineSlot
(
resource		//<p>string<d>Resource value
){
var
/*Object*/outlineSlot =
	{
	resource:	resource,
	type:		this.K.RSC_TYPE.LITERAL
	};
if(this.testMultilineLiteral(resource)){
	outlineSlot.resource = resource.substr(0, resource.length - 1);		//remove trailing MULTILINE_END_CHAR char
	outlineSlot.type = this.K.RSC_TYPE.MULTILINE;
	}
return outlineSlot;
}//</m>getLiteral



//================
this.getLiteralSet = function		//<m>string<a>private<d>Get type and string lang of a literal
(
resource		//<p>string<d>Resource value
){
var
/*Object*/literalSet =
	{
	resource:	resource,
	type:		'string'
	},
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.LITERAL);
if(resourceSet){
	var
	/*string[]*/literalResourceSet = resourceSet[1].split(this.K.RSC_PATTERN.LITERAL_TYPE_CHAR);
	if(literalResourceSet.length > 1) literalSet.lang = literalResourceSet[1];	//e.g. @string:en-US
	else literalSet.type = resourceSet[1];		//e.g. @string
	literalSet.resource = resourceSet[2];
	}
return literalSet;
}//</m>getLiteralSet



//================
this.getOutlineSlot = function		//<m>Object<a>private<d>Get an outlineSet object
(){
return {
		subject:
			{
			resource:	'',
			type:		this.K.RSC_TYPE.UNKNOWN,
			digest:		this.getBNodeDigest(),
			bNode:		0
			},
		predicate:
			{
			resource:	'',
			type:		this.K.RSC_TYPE.UNKNOWN
			},
		object:
			{
			resource:	'',
			type:		this.K.RSC_TYPE.UNKNOWN
			}
		};
}//</m>getOutlineSlot



//================
this.getBNodeDigest = function		//<m>Object<a>private<d>Get the digest of a bNode object
(){
return this.K.SLOT.BNODE + this.formatter.getISODateTime(new Date()) + ':';
}//</m>getBNodeDigest



//================
this.getBNodeResource = function		//<m>string<a>private<d>Get an identifier for a bNode
(){
var
/*Object*/subject = this.outlineSet[this.outlineSet.length - 1].subject,
/*string*/resource = subject.resource,
/*string[]*/resourceSet = resource.match(this.K.RSC_PATTERN.CONTAINER);
if(resourceSet) resource = resourceSet[2];
return resource + subject.digest + subject.bNode++;
}//</m>getBNodeResource



//================
this.testToUpperLevel = function		//<m>boolean<a>private<d>Check whether outline level has changed
(
level			//<p>int<d>Outline level
){
return (level - (this.outlineSet.length - 1)) > 0;
}//</m>testToUpperLevel



//================
this.testBNodeResource = function		//<m>boolean<a>private<d>Check whether resource is anonymous
(
resource		//<p>string<d>Resource value
){
return resource == '';
}//</m>testBNodeResource



//================
this.testMultilineLiteral = function		//<m>boolean<a>private<d>Check whether literal resource is multiline
(
resource		//<p>string<d>Resource value
){
return resource.charAt(resource.length - 1) == this.K.RSC_PATTERN.MULTILINE_END_CHAR;
}//</m>testMultilineLiteral



//================
this.testLiteralType = function		//<m>boolean<a>private<d>Check whether resource type is one of literal
(
type		//<p>int<d>Resource type
){
return type & (this.K.RSC_TYPE.LITERAL | this.K.RSC_TYPE.MULTILINE);
//return type >= this.K.RSC_TYPE.LITERAL;
}//</m>testLiteralType



//================
this.testMultilineLiteralType = function		//<m>boolean<a>private<d>Check whether resource type is mltiline
(
type		//<p>int<d>Resource type
){
return type & this.K.RSC_TYPE.MULTILINE;
//return type == this.K.RSC_TYPE.MULTILINE;
}//</m>testMultilineLiteralType



this.K =
	{
	STEP:
		{
		SUBJECT:	0,
		PREDICATE:	1,
		OBJECT:		2
		},

	SLOT:
		{
		SUBJECT:			'<subject>',
		PREDICATE:			'<predicate>',
		CONTAINER_PREDICATE:'_',
		BNODE:				':bnode:'
		},

	RSC_TYPE:
		{
		UNKNOWN:			0,
		BNODE:				1 << 0,
		RESOURCE:			1 << 1,
		BAG:				1 << 2,
		SEQ:				1 << 3,
		ALT:				1 << 4,
		COLLECTION:			1 << 5,
		LITERAL:			1 << 16,
		MULTILINE:			1 << 17
		},
	
	RSC_PATTERN:
		{
		OBJECT:				/^\(\)\s*(.*)$/,
		CONTAINER:			/^\((Bag|Seq|Alt)\)\s*(.*)$/,
		COLLECTION:			/^\(Collection\)$/,
		LITERAL:			/^@([^\s]+)\s*(.*)$/,
		CONTAINER_ID:		'(%s)',
		LITERAL_TYPE_CHAR:	':',
		MULTILINE_END_CHAR:	'\\'
		},

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		FATAL:					0xFF		//end of error codes
		}
	};



this.ASRDF_TripleParser(_propertyParser);
}//</c>ASRDF_TripleParser

