/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function IO_file_LineIterator		//<c><a>protected<d>Readline iterator class
(
_stream		//<p>XPCOM_nsIScriptableInputStream<d>Input stream instance
){
//================
this.IO_file_LineIterator = function		//<m>void<a>private<d>class constructor
(
stream		//<p>XPCOM_nsIScriptableInputStream<d>Input stream instance
){
this.className	= 'IO_file_LineIterator';
this.stream		= stream;
//i12n
this.errorNotifier = new ERROR_Notifier(this.className);
this.chunk		= this.K.BUFFER_CAPACITY;
this.lineSet	= [];
this.lines		= 0;
this.atLine		= -1;
}//</m>IO_file_LineIterator



//================
this.setUp = function		//<m>boolean<a>private<d>Set up lineSet member with a chunk of lines
(){
try{
	this.set_chunk(this.stream.available());		//read in only one chunk (no buffering)
	var
	/*string*/chunk = this.stream.read(this.chunk);
	//Stdout.setOut('IO_file_LineIterator.setUp chunk length: %N', chunk.length);
	//Stdout.setOut('IO_file_LineIterator.setUp chunk: %N', chunk.substr(0, 32));
	if(chunk == '') return this.K.EOF;
	this.lineSet = this.lineSet.concat(chunk.split(this.K.LINE_SEPARATOR));
	return true;
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.BUFFER_READ);
	return null;
	}
}//</m>setUp



//================
this.set_chunk = function		//<m>void<a>public<d>Set up read buffer capacity
(
chunk		//<p>int<d>Buffer capacity
){
this.chunk = chunk || this.K.BUFFER_CAPACITY;	//avoid undefined capacity
}//</m>set_chunk



//================
this.getNext = function		//<m>string<a>public<d>Get next line from buffer
(
target		//<p>any<d>Number of line to step forward or pattern to match --optional
){
var
/*string*/line = (this.testEmpty() && !this.setUp()) ? this.K.EOF :
	!target ? this.lineSet.shift(this.atLine++) :	//atLine incrementation as shift method argument is just a trick
		(typeof(target) == 'number') ? this.getNextStep(target) :
			this.getNextMatch(target);
if(line != this.K.EOF) this.atLine++;
return line;
}//</m>getNext



//================
this.getNextStep = function		//<m>string<a>public<d>Get next line from buffer stepping to a specified number of lines
(
step		//<p>int=1<d>Number of line to step forward
){
var
/*string*/line = '';
for(var i=0,m=Math.abs(step);(i < m) && (line != this.K.EOF);i++) line = this.getNext();
return line;
}//</m>getNextStep<d>&
/*
Step value is converted to absolute value to avoid infinite looping
*/



//================
this.getNextMatch = function		//<m>string<a>public<d>Get next line from buffer matching a RegExp
(
pattern		//<p>string<d>Pattern of next line to retrieve
){
var
/*string*/line = '';
try{
	pattern = new RegExp(pattern);
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.INVALID_REGEXP);	
	return null;
	}
do{
	line = this.getNext();
	if(line && line.match(pattern)) break;
	}
	while(line != this.K.EOF)
return line;
}//</m>getNextMatch



//================
this.get_atLine = function		//<m>int<a>public<d>Get index of last line retrieved
(){
return this.atLine;
}//</m>get_atLine<d>&
/*
Returns -1 if no line have been retrieved
*/



//================
this.testEmpty = function		//<m>boolean<a>private<d>Check whether lineSet is empty
(){
return this.lineSet.length == 0;
}//</m>testEmpty



this.K =
	{
	BUFFER_CAPACITY:			1024 << 4,	//16 ko buffer (no line length in buffer must be greater than this value)
	EOF:						null,
	LINE_SEPARATOR:				/[\n\r]/g,	//RegExp

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		BUFFER_READ:			-1,
		INVALID_REGEXP:			-2,
		FATAL:					0xFF		//end of error codes
		}
	};



this.IO_file_LineIterator(_stream);
}//</c>IO_file_LineIterator



