/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function RDFAPI_Store		//<c><a>public<d>RDF storage class
(
_storeURL,		//<p>string<d>URL or null for in-memory model
_storage		//<p>int<d>Storage mode according to loading process
){
const ASSERTING = false;
//================
this.RDFAPI_Store = function		//<m>void<a>private<d>class constructor
(
storeURL,	//<p>string<d>URL or null for in-memory model
storage		//<p>int<d>Storage mode according to loading process
){
this.className		= 'RDFAPI_Store';
this.storeURL		= storeURL;
this.storage		= storage;
//i12n
this.errorNotifier	= new ERROR_Notifier(this.className);
this.loadMode		= this.K.BLOCKING_CONNECTION;
this.store			= null;
this.connected		= false;
if(this.setXPCOM() == this.K.ERROR.NONE) this.setUp();	//this.XPCOM_RDFService
}//</m>RDFAPI_Store



//================
this.setXPCOM = function		//<m>int<a>private<d>SetUp RDF service interface
(){
try{
	this.XPCOM_RDFService = XPConnector.getService('nsIRDFService');
	this.XPCOM_RDFSerialiser = XPConnector.getInstance('nsIRDFXMLSerializer');
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.SERVICE_ACCESS);
	}
}//</m>setXPCOM



//================
this.setUp = function		//<m>int<a>public<d>Set up the data source
(){
const
/*int*/
MEMORY_SYNC_STORAGE	= this.K.MEMORY_SYNC_STORAGE,
REMOTE_SYNC_STORAGE = this.K.REMOTE_SYNC_STORAGE,
REMOTE_ASYNC_STORAGE = this.K.REMOTE_ASYNC_STORAGE;
if(!this.testService()) return this.K.ERROR.SERVICE_ACCESS;
var
/*string*/method;
switch(this.storage){
	case MEMORY_SYNC_STORAGE:
		method = 'setInMemoryStore';
		break;
	case REMOTE_SYNC_STORAGE:
		//this.XPCOM_RDFService.GetDataSourceBlocking(this.storeURL);		//NOT IMPLEMENTED
	case REMOTE_ASYNC_STORAGE:
		method = 'setRemoteStore';
		break;
	}	//---switch
this[method]();
}//</m>setUp



//================
this.setInMemoryStore = function		//<m>int<a>private<d>Set up storage member
(){
try{
	this.set_storageFromInstance('nsIRDFInMemoryDataSource');	//set up this.store
	this.store instanceof Components.interfaces.nsIRDFDataSource;
	//this.store = XPConnector.getQuery('nsIRDFDataSource', this.store);
	return this.set_connection('setInMemoryStore');
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.MODEL_LOAD);
	}
}//</m>setInMemoryStore



//================
this.setRemoteStore = function		//<m>int<a>private<d>Set up storage member from remote URL
(){
this.set_storageFromInstance('nsIRDFDataSource');
if(!this.testInit()) this.set_storageFromService();
else this.testRefresh('REPORTING');
try{
	if(this.testLoaded()) return this.set_connection('setRemoteStore 0');
	this.setLoadingObserver();
	//return this.errorNotifier.setOut('**DEBUG** Model loading...', this.K.ERROR.MODEL_LOADING);		//DEBUG purpose
	return this.K.ERROR.MODEL_LOADING;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.MODEL_LOAD);
	}
}//</m>setRemoteStore



//================
this.set_storageFromService = function		//<m>void<a>private<d>Set up storage with RDF service
(){
this.store = this.XPCOM_RDFService.GetDataSource(this.storeURL);
}//</m>set_storageFromService



//================
this.set_storageFromInstance = function		//<m>void<a>private<d>Set up storage with RDF instance
(
instance		//<p>string<d>Interface instance
){
this.store = XPConnector.getInstance(instance);
}//</m>set_storageFromInstance



//================
this.setLoadingObserver = function		//<m>void<a>private<d>Set up storage loading observer
(){
this.loadingObserver = new RDFAPI_StoreObserver(this);
XPConnector.getQuery('nsIRDFXMLSink', this.store).addXMLSinkObserver(this.loadingObserver);
}//</m>setLoadingObserver



//================
this.setHandleObserver = function		//<m>void<a>private<d>Handles observer events
(
notifier		//<p>Object<d>Observer arguments
){
switch(notifier.className){
	case 'RDFAPI_StoreObserver':
		if(notifier.event == 'onEndLoad'){
			notifier.sink.removeXMLSinkObserver(this.loadingObserver);		//take care to remove observer
			return this.set_connection('setHandleObserver');
			}
		else return null;
	case 'RDFAPI_StatementObserver':
		try{
			this.setHandleObserverDelegate(notifier);		//delegate to RDFAPI_Model subclass
			}
		catch(e){
			return;
			}
	default:
		return;
	}	//---switch
}//</m>setHandleObserver



//================
this.set_connection = function		//<m>(boolean)int<a>private<d>Set up the connection member
(
storage		//<p>string<d>Storage
){
this.connected = true;
//return this.errorNotifier.setOut('**DEBUG** Model loaded from ' + storage, this.K.ERROR.NONE);		//DEBUG purpose
return this.K.ERROR.NONE;
}//</m>set_connection<d>&
/*
Method called at the end of the connection process
*/



//================
this.set_loadMode = function		//<m>void<a>public<d>Set up loadMode member
(
mode		//<p>boolean<d>Load mode
){
this.loadMode = mode;
}//</m>set_loadMode



//================
this.setOut = function		//<m>boolean(int)<a>public<d>Output store to a file
(
url		//<p>String<d>Destination URL --optional
){
return !this.testConnected() ? this.K.ERROR.NO_SUCCESS :
	(this.storage == this.K.MEMORY_SYNC_STORAGE) ? this.setMemoryOut(url) :
		this.setFlushOut(url);
}//</m>setOut



//================
this.setFlushOut = function		//<m>boolean(int)<a>public<d>Flush model
(
url		//<p>String<d>Destination URL --optional
){
try{
	this.getRemoteStore()[url ? 'FlushTo' : 'Flush'](url);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.FLUSH_ACCESS);
	}
}//</m>setFlushOut



//================
this.setMemoryOut = function		//<m>boolean(int)<a>public<d>Serialize a store
(
url		//<p>String<d>Destination URL
){
try{
	var
	/*IO_file_fileInstantiator*/file = new IO_file_fileInstantiator();
	if(file.set_handle(url, 0644) != this.K.ERROR.NONE) return this.K.ERROR.NO_SUCCESS;		//FIXME: we should have an alert
	var
	/*Object*/serializer = this.XPCOM_RDFSerialiser;
	/*Object*/output =
		{
		content: '',
		write: function(buffer, count)
			{
			this.content += buffer;
			return count;
			}
		};
	serializer.init(this.store);
	serializer instanceof Components.interfaces.nsIRDFXMLSource;
	serializer.Serialize(output);
	file.setOut(output.content);
	file.set_handle();
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.FLUSH_ACCESS);
	}
}//</m>setOut



//================
this.setRemoveStatementSet = function		//<m>(boolean)int<a>public<d>Remove a collection of previously marked statement
(){
		//FIXME: check 
try{
	this.store().Sweep();
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.REGISTRY_ACCESS);
	}
}//</m>setRemoveStatementSet



//================	FIXME: do we need this ?
this.setResourceRegistry = function		//<m>(boolean)int<a>public<d>Register or Unregister the resource
(
resource,			//<p>Object<d>Resource to compare
registering,		//<p>boolean<d>Register flag: register || unregister
replacing			//<p>boolean<d>Replace previously registered flag: replace || preserve
){
try{
	this.store[registering ? 'RegisterResource' : 'UnregisterResource'](resource, replacing);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.REGISTRY_ACCESS);
	}
}//</m>setResourceRegistry



//================	FIXME: do we need this ?
this.setModelRegistry = function		//<m>(boolean)int<a>public<d>Register or unregister the model
(
registering,		//<p>boolean<d>Register or unregister
replacing			//<p>boolean=false<d>Replace previous model when registering
){
try{
	this.XPCOM_RDFService[registering ? 'RegisterDataSource' : 'UnregisterDataSource'](this.store, replacing);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.REGISTRY_ACCESS);
	}
}//</m>setModelRegistry



//================
this.setObserver = function		//<m>(boolean)int<a>public<d>Set up an observer of the model
(
observer,		//<p>RDFAPI_StatementObserver<d>Observer
registering		//<p>boolean<d>Register or unregister the observer
){
try{
	this.store()[registering ? 'AddObserver' : 'RemoveObserver'](observer);
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.OBSERVER_ACCESS);
	}
}//</m>setObserver



//================
this.get_store = function		//<m>Object<a>private<d>Get store member
(){
return this.store;
}//</m>get_store



//================
this.getRemoteStore = function		//<m>Object<a>private<d>Get remote interface for storage
(){
return XPConnector.getQuery('nsIRDFRemoteDataSource', this.store);
}//</m>getRemoteStore



//================
this.getMarkStore = function		//<m>boolean<a>private<d>Get store for markability
(){
try{
	return XPConnector.getQuery('nsIRDFPurgeableDataSource', this.store);
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.PURGEABLE_ACCESS);
	return false;
	}
}//</m>getMarkStore



//================
this.getSubjectSet = function		//<m>Object<a>public<d>Get an enumeration of all subject resources in store
(){
try{
	return this.store.GetAllResources();
	}
catch(e){
	return null;
	}
}//</m>getSubjectSet



//================
this.getStatementSet = function		//<m>RDFAPI_Triple[]<a>public<d>Get an array of all statements in store
(){
var
/*Object[]*/statementSet = [];
try{
	var
	/*Object*/subjectEnum = this.getSubjectSet();
	for( ; subjectEnum.hasMoreElements(); ) statementSet = statementSet.concat(this.getSubjectStatementSet(subjectEnum.getNext()));
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.STATEMENT_ACCESS);
	}
return statementSet;
}//</m>getStatementSet



//================
this.getArcSet = function		//<m>Object<a>public<d>Get an enumeration of all arcs referring to resource
(
resource,		//<p>Object<d>Resource to test
outcoming		//<p>boolean<d>Direction flag: out || in
){
try{
	return this.store[outcoming ? 'ArcLabelsOut' : 'ArcLabelsIn'](resource);
	}
catch(e){
	return null;
	}
}//</m>getArcSet



//================
this.testCollector = function		//<m>boolean<a>public<d>Check whether resource is a container
(
resource		//<p>RDFAPI_Resource<d>Resource to test
){
try{
	return XPConnector.getService('nsIRDFContainerUtils').IsContainer(this.store, resource);
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.SERVICE_ACCESS);
	return false;
	}
}//</m>testCollector



//================
this.testService = function		//<m>boolean<a>private<d>Check whether RDF service is available
(){
return this.XPCOM_RDFService != null;
}//</m>testService



//================
this.testLoaded = function		//<m>boolean<a>private<d>Check whether model has been loaded
(){
return this.getRemoteStore().loaded;
}//</m>testLoaded



//================
this.testInit = function		//<m>boolean<a>private<d>Try to init remote storage
(){
try{
	this.getRemoteStore().Init(this.storeURL);		//throws model already loaded exception
	return true;
	}
catch(e){
	return false;	//model already loaded
	}
}//</m>testInit



//================
this.testConnected = function		//<m>boolean<a>protected<d>Get connected member to check connection
(){
return this.connected;
}//</m>testConnected



//================
this.testRefresh = function		//<m>boolean<a>private<d>Try to refresh remote model
(
reporting		//<p>boolean<d>Error report flag: report || don't
){
try{
	this.getRemoteStore().Refresh(this.loadMode);
	return true;
	}
catch(e){
	if(reporting) this.errorNotifier.setOut(e, this.K.ERROR.MODEL_LOAD);
	return false;
	}
}//</m>testRefresh



this.K =
	{
	MEMORY_SYNC_STORAGE:			0,
	REMOTE_SYNC_STORAGE:			1,
	REMOTE_ASYNC_STORAGE:			2,
	
	NON_BLOCKING_CONNECTION:	false,
	BLOCKING_CONNECTION:		true,

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,
		MODEL_LOADING:			2,

		//FATAL
		SERVICE_ACCESS:			-1,
		MODEL_LOAD:				-2,
		//COMPOSITE_ACCESS:		-3,
		REGISTRY_ACCESS:		-4,
		FLUSH_ACCESS:			-6,
		PURGEABLE_ACCESS:		-7,
		OBSERVER_ACCESS:		-8,
		STATEMENT_ACCESS:		-9,
		FATAL:					0xFF		//end of error codes
		}
	};


this.RDFAPI_Store(_storeURL, _storage);
}//</c>RDFAPI_Store

